#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "str.h"
#include "checkers.h"

typedef struct location_t{

	char	row;		/* The row in which a piece is located.					*/
	char	column;		/* The column in which a piece is located.				*/

} location_t;

typedef struct movement_t{

	p_color				color;			/* Defines the player making the move.	*/
	struct location_t	origin;			/* Point of origin.						*/
	struct location_t	destination;	/* Point of destination.				*/
	m_type				type;			/* Defines the type of the move.		*/

} movement_t;

typedef struct node{

	struct movement_t	data;	/* Contains movement data.				*/
	struct node*		next;	/* Contains a pointer to the next node.	*/

} node;

typedef struct player_t{

	p_color				color;				/* Defines the player's color.						*/
	struct location_t	pieces[MAX_PIECES];	/* Contains all the pieces a player has.			*/
	list				regulars;			/* Contains all the regular moves a piece can make.	*/
	list				jumps;				/* Contains all the jumps a piece can make.			*/
	p_user				user;				/* Defines the user playing.						*/

} player_t;

typedef struct piece_t{

	p_color	pl;		/* The player to which this piece belongs.						*/
	int		king;	/* This piece is a king: 1 - Yes, 0 - No.						*/
	char	sign;	/* Defines the sighn that represent this piece on the board.	*/

} piece_t;

typedef struct board_t{

	struct piece_t	playing_b[BOARD_SIZE][BOARD_SIZE];	/* Defines a playing board.						*/
	player			p1;									/* Points to player 1.							*/
	player			p2;									/* Points to player 2.							*/
	int				turn;								/* Tracks turn order.							*/
	int				dificulty;							/* Sets dificulty lever for computer player.	*/

} board_t;

typedef	struct gamelog_t{

	list	head;	/* Contains the first node of the list.	*/
	list	tail;	/* Contains the last node of the list.	*/
} gamelog_t;

/* Gets two movement structs and compares them. 1 - Equal, 0 - Different. */
int			movement_compare(movement_t mov1, movement_t mov2){

	int	i;
	int	m1[4], m2[4];

	/* m1 gets all coordinates of mov1. */
	m1[0] = (mov1.origin.row - 'A');
	m1[1] = (mov1.origin.column - '1');
	m1[2] = (mov1.destination.row - 'A');
	m1[3] = (mov1.destination.column - '1');

	/* m2 gets all coordinates of mov2. */
	m2[0] = (mov2.origin.row - 'A');
	m2[1] = (mov2.origin.column - '1');
	m2[2] = (mov2.destination.row - 'A');
	m2[3] = (mov2.destination.column - '1');

	/* Compares coordinates. Returns 0 if different. */
	for (i = 0; i < 4; i++){
		if (m1[i] != m2[i]){
			return 0;
		}
	}

	return 1;	/* Returns 1 if no difference was found. */

} /*movement_compare*/

/*----------------*/
/* List Functions */
/*----------------*/
/* Gets an empty list and creates the first node. */
list		create_list(movement_t mov){

	list new_node;

	new_node = (list)malloc(sizeof(struct node));
	if (!new_node){														/* Checks allocation success.	*/
		printf("Not enough memory. Program is shutting down!\n\n");
		exit(-1);
	}

	new_node->data = mov;		/* Fill node's data. */
	new_node->next = NULL;		/* Initialize node's next. */

	return new_node;

} /*create_list*/

/* Gets a list and adds a node to the list's end. Returns new tail. */
list		add_movement(list tail, movement_t mov){

	list new_node;

	new_node = (list)malloc(sizeof(struct node));
	if (!new_node){														/* Checks allocation success.	*/
		printf("Not enough memory. Program is shutting down!\n\n");
		exit(-1);
	}

	new_node->data = mov;		/* Fill node's data. */	
	new_node->next = NULL;		/* Initialize node's next. */
	tail->next = new_node;		/* Moves tail to last node. */

	return new_node;

} /*add_movement*/

/* Connects a list to the end of another list and returns the new tail. */
list		lists_connect(list tail, list head){

	tail->next = head;		/* Last node on list 1 is linked to the first node of list 2. */

	/* Finds list's tail. */
	while (tail->next != NULL){
		tail = tail->next;
	}

	return tail;

} /*lists_connect*/

/* Gets a list and returns a pointer to the last node. */
list		get_tail(list head){

	/* If list is empty, no tail. */
	if (head == NULL){
		return head;
	}

	/* Searches for the last node. */
	while (head->next != NULL){
		head = head->next;
	}

	return head;

} /*get_tail*/

/* Gets a list of movements and clears it. */
list		clear_list(list head){

	if (head == NULL){		/* End of list. */
		return NULL;
	}

	clear_list(head->next);	/* Recursive call. */
	free(head);				/* Next node was freed, free this one. */

	return NULL;

} /*clear_list*/

/* Gets a movement and returns 1 if found, 0 if not. */
int			find_in_list(movement mov, list head){

	while (head != NULL){
		if (movement_compare(head->data, *mov)){	/* When found: Define move type. */
			mov->type = head->data.type;
			return 1;
		}

		head = head->next;
	}

	return 0;		/* If not found, return 0. */

} /*find_in_list*/

/*-------------------*/
/* General Functions */
/*-------------------*/
/* ~ Internal ~ */
/*--------------*/
/* Gets a game log and prints the last turn played. */
void		print_last_move(gamelog log){

	char	move[4];

	if (log->head == NULL){	/* If no moves were made, print nothing. */
		return;
	}

	/* Move gets the coordinates of the last move. */
	move[0] = log->tail->data.origin.row;
	move[1] = log->tail->data.origin.column;
	move[2] = log->tail->data.destination.row;
	move[3] = log->tail->data.destination.column;

	printf("Last move: %c%c to %c%c.\n", move[0], move[1], move[2], move[3]);

} /*print_last_move*/

/* Gets a board and prints which player should play. */
void		print_turn(board brd){

	if (brd->turn == black)
	{
		printf("Black's Turn:\n\n");
	}
	else{
		printf("White's Turn:\n\n");
	}

} /*print_turn*/

/* Gets a location's coordinates and returns 1 if legal, 0 if not. */
int			legal_location(int* point){

	if ((point[0] < 0) || (point[0] > (BOARD_SIZE -1))){
		return 0;		/* If row exceeds the board, return 0. */
	}

	if ((point[1] < 0) || (point[1] > (BOARD_SIZE - 1))){
		return 0;		/* If column exceeds the board, return 0. */
	}

	return 1;

} /*legal_location*/

/* Gets a position of a piece and resets it to default. */
void		reset_location(location pos){

	pos->row = 0;
	pos->column = 0;

} /*reset_location*/

/* Gets a piece and resets it to default. */
void		reset_piece(piece p){

	p->pl = 0;
	p->king = 0;

} /*reset_piece*/

/*--------------*/
/* ~ External ~ */
/*--------------*/
int			save_log(gamelog log, char* filename){

	FILE*	output;
	list	current = log->head;
	char	choice, color[COLOR_NAME_SIZE+1];

	do{		/* Save confirmation. Runs while no legal choice. */

		printf("Saving will replace existing log. Do you wish to continue?\n");
		printf("1. Yes.\n");
		printf("2. No.\n");
		printf("Your choice: ");
		choice = getchar();
		FLUSH_INPUT;
		printf("\n");

		switch (choice){
			case '1': break;
			case '2':{
				printf("Operation Cancelled.\n\n");
				return 0;
					 }
			default:{
				system("clear");		/* Clear screen. */
				printf("Your choice was illegal.\n");
					}
		}

	} while (choice != '1');

	output = fopen(filename, "w");		/* Checks file oppening failure. */
	if (!output){
		printf("Can't open file. Operation Cancelled.\n");
		return 0;
	}

	while (current != NULL){

		/* Defines the color logged depending on the color making the move. */
		if (current->data.color == 1){
			str_copy(color, "Black");
		}
		else{
			str_copy(color, "White");
		}

		/* '|' is used to allow using getchar without changing the line. */
		fprintf(output, "|%s: %c%c to %c%c.\n", color, current->data.origin.row, current->data.origin.column, current->data.destination.row, current->data.destination.column);

		current = current->next;
	}

	/* '|' is used to allow using getchar without changing the line. */
	fprintf(output, "|PRINT_BOARD\n|PRINT_LOG\n");		/* Board and log printing instructions at end of game. */

	fclose(output);

	return 1;

} /*save_log*/

player		create_player(){

	int		i;
	player	new_player;

	new_player = (player)malloc(sizeof(struct player_t));
	if (!new_player){													/* Checks allocation success. */
		return NULL;
	}

	for (i = 0; i < MAX_PIECES; i++){	/* No pieces.			*/
		reset_location(new_player->pieces+i);
	}
	new_player->regulars = NULL;		/* No moves.			*/
	new_player->jumps = NULL;			/* No moves.			*/
	new_player->user = human;			/* Default is human.	*/

	return new_player;

} /*create_player*/

board		create_board(){

	int		i, j;
	board	new_board;

	new_board = (board)malloc(sizeof(struct board_t));
	if (!new_board){													/* Checks allocation success. */
		return NULL;
	}

	for (i = 0; i < BOARD_SIZE; i++){									/* Initializes the whole board. */
		for (j = 0; j < BOARD_SIZE; j++){
			new_board->playing_b[i][j].pl = 0;
			new_board->playing_b[i][j].king = 0;
			if ((i%2) == (j%2)){
				new_board->playing_b[i][j].sign = ' ';					/* When both are even or odd - White square. */
			}
			else{
				new_board->playing_b[i][j].sign = ':';					/* When i and j differ - Black square. */
			}
		}
	}

	new_board->turn = black;											/* Black plays first. */
	new_board->dificulty = 1;											/* Default dificulty is 1. */

	return new_board;

} /*create_board*/

gamelog		create_log(){

	gamelog new_log;

	new_log = (gamelog)malloc(sizeof(struct gamelog_t));
	if (!new_log){														/* Checks allocation success. */
		return NULL;
	}

	new_log->head = NULL;		/* Reset head. */
	new_log->tail = NULL;		/* Reset tail. */

	return new_log;

} /*create_log*/

void		set_player(player pl, p_user u){

	pl->user = u;

} /*set_player*/

void		set_dificulty(board brd, int dificulty){

	brd->dificulty = dificulty;

} /*set dificulty*/

void		destroy_player(player pl){

	if (pl == NULL){
		return;
	}

	free(pl);

} /*destroy_player*/

void		destroy_board(board brd){

	if (brd == NULL){
		return;
	}

	free(brd);

} /*destroy_board*/

void		destroy_log(gamelog log){

	if (log == NULL){
		return;
	}

	log->head = clear_list(log->head);
	free(log);

} /*destroy_log*/

/*--------------------*/
/* Movement Functions */
/*--------------------*/
/* ~ Internal ~ */
/*--------------*/
/* Calculates all moves available from a specific position for a regular piece. */
list		calc_mov_reg(location pos, board brd){

	list		head = NULL, tail = NULL;
	movement_t	mov;
	int			m[4];	/* Contains movement coordinates. */

	/* Origin gets current piece position. */
	mov.origin.row = pos->row;
	mov.origin.column = pos->column;
	m[0] = (pos->row - 'A');
	m[1] = (pos->column - '1');
	mov.color = brd->playing_b[m[0]][m[1]].pl;			/* Move belongs the piece controller. */

	m[2] = (m[0] + (1*mov.color));						/* Move one row. Direction depends on the piece's color. */
	m[3] = (m[1] - (1*mov.color));						/* Move one column. Direction depends on the piece's color. */
	if (legal_location(m+2)){							/* Makes sure the location is legal. */
		if (brd->playing_b[m[2]][m[3]].pl == 0){		/* Makes sure the destination is empty. */
			mov.destination.row = (m[2] + 'A');
			mov.destination.column = (m[3] + '1');
			mov.type = move;							/* Defines as move. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	m[2] = (m[0] + (1*mov.color));						/* Move one row. Direction depends on the piece's color. */
	m[3] = (m[1] + (1*mov.color));						/* Move one column. Direction depends on the piece's color. */
	if (legal_location(m+2)){							/* Makes sure the location is legal. */
		if (brd->playing_b[m[2]][m[3]].pl == 0){		/* Makes sure the destination is empty. */
			mov.destination.row = (m[2] + 'A');
			mov.destination.column = (m[3] + '1');
			mov.type = move;							/* Defines as move. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	return head;

} /*calc_mov_reg*/

/* Calculates all moves available from a specific position for a king piece. */
list		calc_mov_king(location pos, board brd){

	list		head = NULL, tail = NULL;
	movement_t	mov;
	int			m[4], i;	/* Contains movement coordinates. */

	/* Origin gets current piece position. */
	mov.origin.row = pos->row;
	mov.origin.column = pos->column;
	m[0] = (pos->row - 'A');
	m[1] = (pos->column - '1');
	mov.color = brd->playing_b[m[0]][m[1]].pl;				/* Move belongs the piece controller. */

	/* Checks diagonal 1. */
	for (i = 1; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[2] = (m[0] + (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[3] = (m[1] + (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		if (!legal_location(m+2)){							/* stops if destination is not on the board. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl != 0){			/* Stops if destination is not empty. */
			break;
		}

		mov.destination.row = (m[2] + 'A');
		mov.destination.column = (m[3] + '1');
		mov.type = move;							/* Defines as move. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}
	}

	/* Checks diagonal 2. */
	for (i = 1; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[2] = (m[0] - (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[3] = (m[1] - (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		if (!legal_location(m+2)){							/* stops if destination is not on the board. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl != 0){			/* Stops if destination is not empty. */
			break;
		}

		mov.destination.row = (m[2] + 'A');
		mov.destination.column = (m[3] + '1');
		mov.type = move;							/* Defines as move. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}
	}

	/* Checks diagonal 3. */
	for (i = 1; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[2] = (m[0] + (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[3] = (m[1] - (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		if (!legal_location(m+2)){							/* stops if destination is not on the board. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl != 0){			/* Stops if destination is not empty. */
			break;
		}

		mov.destination.row = (m[2] + 'A');
		mov.destination.column = (m[3] + '1');
		mov.type = move;							/* Defines as move. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}
	}

	/* Checks diagonal 4. */
	for (i = 1; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[2] = (m[0] - (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[3] = (m[1] + (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		if (!legal_location(m+2)){							/* stops if destination is not on the board. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl != 0){			/* Stops if destination is not empty. */
			break;
		}

		mov.destination.row = (m[2] + 'A');
		mov.destination.column = (m[3] + '1');
		mov.type = move;							/* Defines as move. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}
	}

	return head;

} /*calc_mov_king*/

/* Calculates all jumps available from a specific position for a regular piece. */
list		calc_jumps_reg(location pos, board brd){

	list		head = NULL, tail = NULL;
	movement_t	mov;
	int			m[6];	/* Contains movement coordinates. */

	/* Origin gets current piece position. */
	mov.origin.row = pos->row;
	mov.origin.column = pos->column;
	m[0] = (pos->row - 'A');
	m[1] = (pos->column - '1');
	mov.color = brd->playing_b[m[0]][m[1]].pl;	/* Move belongs the piece controller. */

	m[4] = (m[0] + (2*mov.color));						/* Move two rows. Direction depends on the piece's color. */
	m[5] = (m[1] - (2*mov.color));						/* Move two columns. Direction depends on the piece's color. */
	m[2] = (m[4] - (1*mov.color));						/* Gets row before destination. */
	m[3] = (m[5] + (1*mov.color));						/* Gets column before destination. */
	if ((legal_location(m+4) && (brd->playing_b[m[2]][m[3]].pl != 0))){										/* Makes sure the location is legal and there is a piece to jump over. */
		if ((brd->playing_b[m[2]][m[3]].pl == (mov.color*-1)) && (brd->playing_b[m[4]][m[5]].pl == 0)){		/* Makes sure the piece we jump over belongs to the other player and destination is empty. */
			mov.destination.row = (m[4] + 'A');
			mov.destination.column = (m[5] + '1');
			mov.type = jump;							/* Defines as jump. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	m[4] = (m[0] + (2*mov.color));						/* Move two rows. Direction depends on the piece's color. */
	m[5] = (m[1] + (2*mov.color));						/* Move two columns. Direction depends on the piece's color. */
	m[2] = (m[4] - (1*mov.color));						/* Gets row before destination. */
	m[3] = (m[5] - (1*mov.color));						/* Gets column before destination. */
	if ((legal_location(m+4) && (brd->playing_b[m[2]][m[3]].pl != 0))){										/* Makes sure the location is legal and there is a piece to jump over. */
		if ((brd->playing_b[m[2]][m[3]].pl == (mov.color*-1)) && (brd->playing_b[m[4]][m[5]].pl == 0)){		/* Makes sure the piece we jump over belongs to the other player and destination is empty. */
			mov.destination.row = (m[4] + 'A');
			mov.destination.column = (m[5] + '1');
			mov.type = jump;							/* Defines as jump. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	return head;

} /*calc_jumps_reg*/

/* Calculates all jumps available from a specific position for a king piece. */
list		calc_jumps_king(location pos, board brd){

	list		head = NULL, tail = NULL;
	movement_t	mov;
	int			m[6], i;	/* Contains movement coordinates. */

	/* Origin gets current piece position. */
	mov.origin.row = pos->row;
	mov.origin.column = pos->column;
	m[0] = (pos->row - 'A');
	m[1] = (pos->column - '1');
	mov.color = brd->playing_b[m[0]][m[1]].pl;				/* Move belongs the piece controller. */

	/* Checks diagonal 1. */
	for (i = 2; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[4] = (m[0] + (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[5] = (m[1] + (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[2] = (m[4] - (1*mov.color));						/* Gets row before destination. */
		m[3] = (m[5] - (1*mov.color));						/* Gets column before destination. */
		if (!legal_location(m+4)){							/* stops if destination is not on the board. */
			break;
		}
		if ((brd->playing_b[m[4]][m[5]].pl != 0) && (brd->playing_b[m[2]][m[3]].pl != 0)){	/* Stops if encounters two adjacent pieces. */
			break;
		}
		if (brd->playing_b[m[4]][m[5]].pl == mov.color){	/* Stops if encounters own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == mov.color){	/* Stops if jumped over own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == 0){			/* Moves to next iteration if jumped over an empty location. */
			continue;
		}
		if (brd->playing_b[m[4]][m[5]].pl != 0){			/* Moves to next iteration if destination is not empty. */
			continue;
		}

		mov.destination.row = (m[4] + 'A');
		mov.destination.column = (m[5] + '1');
		mov.type = jump;							/* Defines as jump. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}

		break;										/* If a jump was found, stop looking in this diagonal. */
	}

	/* Checks diagonal 2. */
	for (i = 2; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[4] = (m[0] - (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[5] = (m[1] - (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[2] = (m[4] + (1*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[3] = (m[5] + (1*mov.color));						/* Gets column before destination. */
		if (!legal_location(m+4)){							/* stops if destination is not on the board. */
			break;
		}
		if ((brd->playing_b[m[4]][m[5]].pl != 0) && (brd->playing_b[m[2]][m[3]].pl != 0)){	/* Stops if encounters two adjacent pieces. */
			break;
		}
		if (brd->playing_b[m[4]][m[5]].pl == mov.color){	/* Stops if encounters own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == mov.color){	/* Stops if jumped over own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == 0){			/* Moves to next iteration if jumped over an empty location. */
			continue;
		}
		if (brd->playing_b[m[4]][m[5]].pl != 0){			/* Moves to next iteration if destination is not empty. */
			continue;
		}

		mov.destination.row = (m[4] + 'A');
		mov.destination.column = (m[5] + '1');
		mov.type = jump;							/* Defines as jump. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}

		break;										/* If a jump was found, stop looking in this diagonal. */
	}

	/* Checks diagonal 3. */
	for (i = 2; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[4] = (m[0] + (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[5] = (m[1] - (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[2] = (m[4] - (1*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[3] = (m[5] + (1*mov.color));						/* Gets column before destination. */
		if (!legal_location(m+4)){							/* stops if destination is not on the board. */
			break;
		}
		if ((brd->playing_b[m[4]][m[5]].pl != 0) && (brd->playing_b[m[2]][m[3]].pl != 0)){	/* Stops if encounters two adjacent pieces. */
			break;
		}
		if (brd->playing_b[m[4]][m[5]].pl == mov.color){	/* Stops if encounters own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == mov.color){	/* Stops if jumped over own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == 0){			/* Moves to next iteration if jumped over an empty location. */
			continue;
		}
		if (brd->playing_b[m[4]][m[5]].pl != 0){			/* Moves to next iteration if destination is not empty. */
			continue;
		}

		mov.destination.row = (m[4] + 'A');
		mov.destination.column = (m[5] + '1');
		mov.type = jump;							/* Defines as jump. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}

		break;										/* If a jump was found, stop looking in this diagonal. */
	}

	/* Checks diagonal 4. */
	for (i = 2; i < BOARD_SIZE; i++){						/* Runs on a diagonal line. */
		m[4] = (m[0] - (i*mov.color));						/* Move i rows. Direction depends on the piece's color. */
		m[5] = (m[1] + (i*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[2] = (m[4] + (1*mov.color));						/* Move i columns. Direction depends on the piece's color. */
		m[3] = (m[5] - (1*mov.color));						/* Gets column before destination. */
		if (!legal_location(m+4)){							/* stops if destination is not on the board. */
			break;
		}
		if ((brd->playing_b[m[4]][m[5]].pl != 0) && (brd->playing_b[m[2]][m[3]].pl != 0)){	/* Stops if encounters two adjacent pieces. */
			break;
		}
		if (brd->playing_b[m[4]][m[5]].pl == mov.color){	/* Stops if encounters own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == mov.color){	/* Stops if jumped over own piece. */
			break;
		}
		if (brd->playing_b[m[2]][m[3]].pl == 0){			/* Moves to next iteration if jumped over an empty location. */
			continue;
		}
		if (brd->playing_b[m[4]][m[5]].pl != 0){			/* Moves to next iteration if destination is not empty. */
			continue;
		}

		mov.destination.row = (m[4] + 'A');
		mov.destination.column = (m[5] + '1');
		mov.type = jump;							/* Defines as jump. */

		if (head == NULL){							/* Adds to a list of possible movement for this piece. */
			head = create_list(mov);
			tail = head;
		}
		else{
			tail = add_movement(tail, mov);
		}

		break;										/* If a jump was found, stop looking in this diagonal. */
	}

	return head;

} /*calc_jumps_king*/

/* Calculates all jumps available from a specific position for a regular piece, after a jump took place. */
list		jump_again_reg(location pos, board brd){

	list		head = NULL, tail = NULL;
	movement_t	mov;
	int			m[6];	/* Contains movement coordinates. */

	/* Origin gets current piece position. */
	mov.origin.row = pos->row;
	mov.origin.column = pos->column;
	m[0] = (pos->row - 'A');
	m[1] = (pos->column - '1');
	mov.color = brd->playing_b[m[0]][m[1]].pl;	/* Move belongs the piece controller. */

	m[4] = (m[0] + (2*mov.color));						/* Move two rows. Direction depends on the piece's color. */
	m[5] = (m[1] - (2*mov.color));						/* Move two columns. Direction depends on the piece's color. */
	m[2] = (m[4] - (1*mov.color));						/* Gets row before destination. */
	m[3] = (m[5] + (1*mov.color));						/* Gets column before destination. */
	if ((legal_location(m+4) && (brd->playing_b[m[2]][m[3]].pl != 0))){										/* Makes sure the location is legal and there is a piece to jump over. */
		if ((brd->playing_b[m[2]][m[3]].pl == (mov.color*-1)) && (brd->playing_b[m[4]][m[5]].pl == 0)){		/* Makes sure the piece we jump over belongs to the other player and destination is empty. */
			mov.destination.row = (m[4] + 'A');
			mov.destination.column = (m[5] + '1');
			mov.type = jump;							/* Defines as jump. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	m[4] = (m[0] + (2*mov.color));						/* Move two rows. Direction depends on the piece's color. */
	m[5] = (m[1] + (2*mov.color));						/* Move two columns. Direction depends on the piece's color. */
	m[2] = (m[4] - (1*mov.color));						/* Gets row before destination. */
	m[3] = (m[5] - (1*mov.color));						/* Gets column before destination. */
	if ((legal_location(m+4) && (brd->playing_b[m[2]][m[3]].pl != 0))){										/* Makes sure the location is legal and there is a piece to jump over. */
		if ((brd->playing_b[m[2]][m[3]].pl == (mov.color*-1)) && (brd->playing_b[m[4]][m[5]].pl == 0)){		/* Makes sure the piece we jump over belongs to the other player and destination is empty. */
			mov.destination.row = (m[4] + 'A');
			mov.destination.column = (m[5] + '1');
			mov.type = jump;							/* Defines as jump. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	m[4] = (m[0] - (2*mov.color));						/* Move two rows. Direction depends on the piece's color. */
	m[5] = (m[1] + (2*mov.color));						/* Move two columns. Direction depends on the piece's color. */
	m[2] = (m[4] + (1*mov.color));						/* Gets row before destination. */
	m[3] = (m[5] - (1*mov.color));						/* Gets column before destination. */
	if ((legal_location(m+4) && (brd->playing_b[m[2]][m[3]].pl != 0))){										/* Makes sure the location is legal and there is a piece to jump over. */
		if ((brd->playing_b[m[2]][m[3]].pl == (mov.color*-1)) && (brd->playing_b[m[4]][m[5]].pl == 0)){		/* Makes sure the piece we jump over belongs to the other player and destination is empty. */
			mov.destination.row = (m[4] + 'A');
			mov.destination.column = (m[5] + '1');
			mov.type = jump;							/* Defines as jump. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	m[4] = (m[0] - (2*mov.color));						/* Move two rows. Direction depends on the piece's color. */
	m[5] = (m[1] - (2*mov.color));						/* Move two columns. Direction depends on the piece's color. */
	m[2] = (m[4] + (1*mov.color));						/* Gets row before destination. */
	m[3] = (m[5] + (1*mov.color));						/* Gets column before destination. */
	if ((legal_location(m+4) && (brd->playing_b[m[2]][m[3]].pl != 0))){										/* Makes sure the location is legal and there is a piece to jump over. */
		if ((brd->playing_b[m[2]][m[3]].pl == (mov.color*-1)) && (brd->playing_b[m[4]][m[5]].pl == 0)){		/* Makes sure the piece we jump over belongs to the other player and destination is empty. */
			mov.destination.row = (m[4] + 'A');
			mov.destination.column = (m[5] + '1');
			mov.type = jump;							/* Defines as jump. */

			if (head == NULL){							/* Adds to a list of possible movement for this piece. */
				head = create_list(mov);
				tail = head;
			}
			else{
				tail = add_movement(tail, mov);
			}
		}
	}

	return head;

} /*jump_again_reg*/

/* Returnes all movements possible for a piece in a specific location. */
list		calculate_movements(location pos, board brd){

	/* Desides how to calcute, depending king or regular piece. */
	if (brd->playing_b[(pos->row) - 'A'][(pos->column) - '1'].king){
		return calc_mov_king(pos, brd);
	}
	else{
		return calc_mov_reg(pos, brd);
	}

} /*calculate_movements*/

/* Returnes all jumps possible for a piece in a specific location. */
list		calculate_jumps(location pos, board brd){

	/* Desides how to calcute, depending king or regular piece. */
	if (brd->playing_b[(pos->row) - 'A'][(pos->column) - '1'].king){
		return calc_jumps_king(pos, brd);
	}
	else{
		return calc_jumps_reg(pos, brd);
	}

} /*calculate_jumps*/

/* Gets a board and a position after a jump and checks for additional jumps (Now including bacwards). */
list		jump_again(location pos, board brd){

	/* Desides how to calcute, depending king or regular piece. */
	if (brd->playing_b[(pos->row) - 'A'][(pos->column) - '1'].king){
		return calc_jumps_king(pos, brd);	/* Kings jump the same as first jump or consecutive. */
	}
	else{
		return jump_again_reg(pos, brd);
	}

} /*jump_again*/

/* Gets a movement and a board and returns 1 if legal, 0 if not. */
int			legal_move(movement mov, board brd){

	player	pl;

	/* Desides player depending on turn. */
	if (brd->turn == brd->p1->color)
	{
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}

	/* Makes sure the origin has a piece. */
	if (brd->playing_b[mov->origin.row - 'A'][mov->origin.column - '1'].pl == 0){
		return 0;
	}

	/* Makes sure the piece's color mathes the turn. */
	if (brd->playing_b[mov->origin.row - 'A'][mov->origin.column - '1'].pl != brd->turn){
		return 0;
	}

	/* Checks if existsts in players optional jumps. */
	if (pl->jumps != NULL){
		return find_in_list(mov, pl->jumps);
	}

	/* If no legal jumps, checks if existsts in players optional moves. */
	if (pl->regulars != NULL){
		return find_in_list(mov, pl->regulars);
	}

	return 0;

} /*legal_move*/

/* Gets a board and returns a game state: 1 - p1 wins, -1 - p2 wins, 0 - no desicion. */
int			game_state(board brd){

	player	pl;

	/* Desides player depending on turn. */
	if (brd->turn == brd->p1->color)
	{
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}

	/* If a player cant move - Return opponent as winner. */
	if ((pl->regulars == NULL) && (pl->jumps == NULL)){
		return (-1*pl->color);
	}

	return 0;

} /*game_state*/

/* Calculates all possible movements for a player. */
int			calculate_all(player pl, board brd){
	
	int		i;
	list	r_tail, j_tail, new_moves;

	/* Reset previous optional movements. */
	pl->regulars = clear_list(pl->regulars);
	pl->jumps = clear_list(pl->jumps);

	/* Check all pieces of this player. */
	for (i = 0; i < MAX_PIECES; i++){
		if (pl->pieces[i].row == 0){	/* If piece is not on the board, go to next piece. */
			continue;
		}

		/* Get all possible jumps for this piece. */
		new_moves = calculate_jumps(pl->pieces+i, brd);

		/* Add jumps to the player's possible movements. */
		if (pl->jumps == NULL)
		{
			pl->jumps = new_moves;
			j_tail = get_tail(pl->jumps);
		}
		else{
			j_tail = lists_connect(j_tail, new_moves);
		}

		if (pl->jumps == NULL){		/* Jumps are mandatory. Checks for regular moves only if no jumps available. */
			/* Get all possible movements for this piece. */
			new_moves = calculate_movements(pl->pieces+i, brd);

			/* Add movements to the player's possible movements. */
			if (pl->regulars == NULL)
			{
				pl->regulars = new_moves;
				r_tail = get_tail(pl->regulars);
			}
			else{
				r_tail = lists_connect(r_tail, new_moves);
			}
		}
	}

	return game_state(brd);

} /*calculate_all*/

/* Gets a movement from the user. Returns 0 when movement is legal, 94 if player wishes to end the game. */
int			get_movement(board brd, movement mov, gamelog log){

	char		m[2];
	int			point[2];

	system("clear");		/* Clear screen. */
	print_last_move(log);
   	print_turn(brd);
	print_board(brd);

	do{		/* Runs while no legal move was entered. */

		printf("Please enter the location of the piece you wish to move: ");

		m[0] = getchar();
		m[0] = upper_char(m[0]);
		if (m[0] == 'Q'){
			return 94;		/* If 'Q' was pressed - Special action: End game.*/
		}

		m[1] = getchar();
		m[1] = upper_char(m[1]);
		if (m[1] == 'Q'){
			return 94;		/* If 'Q' was pressed - Special action: End game.*/
		}

		FLUSH_INPUT;

		point[0] = (m[0] - 'A');
		point[1] = (m[1] - '1');

		/* Makes sure the origin is on the board. */
		if (!legal_location(point)){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("Your choice is not on the board.\n");
			continue;
		}

		/* Makes sure the origin has a piece. */
		if (brd->playing_b[m[0] - 'A'][m[1] - '1'].pl == 0){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("The square you entered is empty.\n");
			continue;
		}

		/* Makes sure the piece's color mathes the turn. */
		if (brd->playing_b[m[0] - 'A'][m[1] - '1'].pl != brd->turn){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("The piece you chose doesn't belong to you.\n");
			continue;
		}

		mov->origin.row = m[0];
		mov->origin.column = m[1];

		printf("Please enter the destination of the move: ");

		m[0] = getchar();
		m[0] = upper_char(m[0]);
		if (m[0] == 'Q'){
			return 94;		/* If 'Q' was pressed - Special action: End game.*/
		}

		m[1] = getchar();
		m[1] = upper_char(m[1]);
		if (m[1] == 'Q'){
			return 94;		/* If 'Q' was pressed - Special action: End game.*/
		}

		FLUSH_INPUT;

		point[0] = (m[0] - 'A');
		point[1] = (m[1] - '1');

		/* Makes sure the origin is on the board. */
		if (!legal_location(point)){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("Your choice is not on the board.\n");
			continue;
		}

		/* Makes sure the destination is empty. */
		if (brd->playing_b[m[0] - 'A'][m[1] - '1'].pl != 0){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("The square you entered is not empty.\n");
			continue;
		}

		/* Makes sure the move remains on 'black' squares. */
		if (brd->playing_b[m[0] - 'A'][m[1] - '1'].sign == ' '){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("You can move only diagonally.\n");
			continue;
		}

		
		mov->destination.row = m[0];
		mov->destination.column = m[1];

		/* Makes sure the move is legal. */
		if (!legal_move(mov, brd)){
			system("clear");		/* Clear screen. */
			print_last_move(log);
			print_turn(brd);
			print_board(brd);
			printf("\n");
			printf("This move is against the rules.\n");
			continue;
		}

		system("clear");		/* Clear screen. */

		break;

	} while (1);

	/* Move belongs the turn controller. */
	mov->color = brd->turn;

	return 0;

} /*get_movement*/

/* Gets a file a board and movement. Edits the movement and returns a special action number. 0 if legal movement. */
int			get_move_f(board brd, movement mov, char* filename){

	char			color[COLOR_NAME_SIZE+1];
	char*			mov_line;
	int				i, point[2];
	static FILE*	input;				/* Static to remember use when exiting the function. */
	static int		file_opened = 0;	/* Static to remember use when exiting the function. */

	system("clear");		/* Clear screen. */
	print_turn(brd);
	print_board(brd);

	/* Checks if file was opened previously. */
	if (!file_opened){
		input = fopen(filename, "r");
		if (!input){		/* Checks file oppening failure. */
			return 99;
		}
		file_opened = 1;	/* Notes file is open. */
	}

	/* Checks for EOF. */
	if (getc(input) == EOF){
		fclose(input);		/* When EOF is reached, close file. */
		file_opened = 0;	/* Notes file is no longer open. */

		return 100;
	}

	mov_line = get_str_f(input);

	/* Checks for print_board and print_log instructions. */
	if (str_compare(mov_line, "PRINT_BOARD") == 0){
		return 91;
	}
	if (str_compare(mov_line, "PRINT_LOG") == 0){
		return 92;
	}

	/* Gets move color from file. */
	for (i = 0; i < COLOR_NAME_SIZE; i++){
		color[i] = mov_line[i];
	}
	color[i] = '\0';
	capitalize(color);
	if (str_compare(color, "Black") == 0){
		mov->color = black;
	}
	else{
		if (str_compare(color, "White") == 0){
			mov->color = white;
		}
		else{
			return 98;
		}
	}

	/* Gets origin and checks if legal presentation of point and if on board. */
	i += 2;
	point[0] = (mov_line[i++] - 'A');
	point[1] = (mov_line[i] - '1');
	if ((!is_letter(point[0] + 'A')) || (!is_digit(point[1] + '1'))){
		fclose(input);		/* If an error occurs, terminate simulation and close file. */
		file_opened = 0;	/* Notes file is no longer open. */
		return 98;
	}
	if (!legal_location(point)){
		fclose(input);		/* If an error occurs, terminate simulation and close file. */
		file_opened = 0;	/* Notes file is no longer open. */
		return 96;
	}
	mov->origin.row = (point[0] + 'A');
	mov->origin.column = (point[1] + '1');

	/* Gets destination and checks if legal presentation of point and if on board. */
	i += 5;
	point[0] = (mov_line[i++] - 'A');
	point[1] = (mov_line[i] - '1');
	if ((!is_letter(point[0] + 'A')) || (!is_digit(point[1] + '1'))){
		fclose(input);		/* If an error occurs, terminate simulation and close file. */
		file_opened = 0;	/* Notes file is no longer open. */
		return 98;
	}
	if (!legal_location(point)){
		fclose(input);		/* If an error occurs, terminate simulation and close file. */
		file_opened = 0;	/* Notes file is no longer open. */
		return 96;
	}
	mov->destination.row = (point[0] + 'A');
	mov->destination.column = (point[1] + '1');

	/* Makes sure the move is legal. */
	if (!legal_move(mov, brd)){
		fclose(input);		/* If an error occurs, terminate simulation and close file. */
		file_opened = 0;	/* Notes file is no longer open. */
		return 97;
	}

	return 0;

} /*get_move_f*/

/*--------------------*/
/* Computer Functions */
/*--------------------*/
/* Gets a range and returns a random number from that range. */
int			random_num(int range){

	srand(time(NULL));	/* Initializes the seed used by rand according to current time (Virtually random). */

	return (rand() % range);	/* Uses the seed to generate a random number. Mod range returns an int between 0 and range. */

} /*random_num*/

/* Gets a movement and a board and executes the move as played by the computer. Returns the utility of the move. */
int			comp_execute(movement_t mov, board brd){

	list		current;
	player		pl;
	location_t	pos;
	int			m[4], removed[2], i, utility = 0;

	/* m gets coordinations of the movement. */
	m[0] = (mov.origin.row - 'A');
	m[1] = (mov.origin.column - '1');
	m[2] = (mov.destination.row - 'A');
	m[3] = (mov.destination.column - '1');

	/* Moves piece from origin to destination and resets origin. */
	brd->playing_b[m[2]][m[3]] = brd->playing_b[m[0]][m[1]];
	brd->playing_b[m[0]][m[1]].pl = 0;
	brd->playing_b[m[0]][m[1]].king = 0;
	brd->playing_b[m[0]][m[1]].sign = ':';

	/* Changes piece's location on the player's pieces' list. */
	if (brd->p1->color == brd->playing_b[m[2]][m[3]].pl){
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}
	for (i = 0; i < MAX_PIECES; i++){
		if ((pl->pieces[i].row == (m[0] + 'A')) && (pl->pieces[i].column == (m[1] + '1'))){
			pl->pieces[i].row = (m[2] + 'A');
			pl->pieces[i].column = (m[3] + '1');
			break;
		}
	}

	/* Special actions after jump. */
	if (mov.type == jump){

		utility += 5*brd->dificulty;		/* Ajump has a utility while a regular movement doesn't. Sooner moves get better utility. */

		/* Gets the location for the removed piece. */
		if (m[2] > m[0]){
			removed[0] = (m[2] - 1);
		}
		else{
			removed[0] = (m[2] + 1);
		}
		if (m[3] > m[1]){
			removed[1] = (m[3] - 1);
		}
		else{
			removed[1] = (m[3] + 1);
		}

		/* Removes piece from player's list. */
		if (brd->p1->color == brd->playing_b[m[2]][m[3]].pl){
			pl = brd->p1;
		}
		else{
			pl = brd->p2;
		}
		for (i = 0; i < MAX_PIECES; i++){
			if ((pl->pieces[i].row == (removed[0] + 'A')) && (pl->pieces[i].column == (removed[1] + '1'))){
				reset_location(pl->pieces+i);
				break;
			}
		}

		/* Resets the removed piece location. */
		if (brd->playing_b[removed[0]][removed[1]].king){	/* If the removed piece is a king, it gets more utility. */
			utility += 10*brd->dificulty;	/* Sooner moves get better utility. */
		}
		brd->playing_b[removed[0]][removed[1]].pl = 0;
		brd->playing_b[removed[0]][removed[1]].king = 0;
		brd->playing_b[removed[0]][removed[1]].sign = ':';

		/* Checks for consecutive jumps.*/
		if (brd->p1->color == brd->playing_b[m[2]][m[3]].pl){
			pl = brd->p1;
		}
		else{
			pl = brd->p2;
		}
		pos.row = mov.destination.row;
		pos.column = mov.destination.column;
		pl->jumps = jump_again(&pos, brd);
		pl->regulars = NULL;
		if (pl->jumps){		/* If exist, gets move from player. */
			current = pl->jumps;
			while (current != NULL){
				utility += comp_execute(current->data, brd);	/* Consecutive jumps have good utility. */
				current = current->next;
			}
		}
	}

	/* If piece is not a king, and moving towards opponent will exceed the board - Piece gets king. */
	if (!brd->playing_b[m[2]][m[3]].king && (((m[2] + (1*mov.color)) < 0) || ((m[2] + (1*mov.color)) > (BOARD_SIZE - 1)))){
		brd->playing_b[m[2]][m[3]].king = 1;
		brd->playing_b[m[2]][m[3]].sign = upper_char(brd->playing_b[m[2]][m[3]].sign);
		utility += 15*brd->dificulty;	/* Making a king has good utility. Sooner moves get better utility. */
	}

	return utility;

} /*comp_execute*/

/* Gets a board and plays one turn played by the computer. */
int			comp_turn(board brd, movement_t mov, int* utility){

	player		pl;
	int			state;

	/* Desides player depending on turn. */
	if (brd->turn == brd->p1->color)
	{
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}

	/* Check for all movements of this player and checks if there is a winner. */
	state = calculate_all(pl, brd);

	/* If no winner, execute movement and calculate its utility. */
	if (!state){
		(*utility) += comp_execute(mov, brd);
	}

	return state;

} /*comp_turn*/

/* Gets a move and a board and calculates its utility. */
int			calc_utility(movement_t mov, board brd){

	board	temp_board;
	player	pl;
	list	current;
	int		i, j, state, utility = 0;

	/* Recursive end condition. */
	if (brd->dificulty == 0){
		return 0;
	}

	/* Copy the board to prevent real board corruption. */
	temp_board = create_board();
	if (!temp_board){
		return 0;	/* If failed to create temp board - Skip calculating utility. */
	}
	temp_board->p1 = create_player();
	temp_board->p2 = create_player();
	if (!temp_board->p1 || !temp_board->p2){
		return 0;	/* If failed to create players - Skip calculating utility. */
	}

	for (i = 0; i < BOARD_SIZE; i++){
		for(j = 0; j < BOARD_SIZE; j++){
			temp_board->playing_b[i][j].pl = brd->playing_b[i][j].pl;
			temp_board->playing_b[i][j].king = brd->playing_b[i][j].king;
			temp_board->playing_b[i][j].sign = brd->playing_b[i][j].sign;
		}
	}
	temp_board->p1->color = brd->p1->color;
	for (i = 0; i < MAX_PIECES; i++){
		temp_board->p1->pieces[i].row = brd->p1->pieces[i].row;
		temp_board->p1->pieces[i].column = brd->p1->pieces[i].column;
	}
	temp_board->p1->regulars = NULL;
	temp_board->p1->jumps = NULL;
	temp_board->p1->user = brd->p1->user;
	temp_board->p2->color = brd->p2->color;
	for (i = 0; i < MAX_PIECES; i++){
		temp_board->p2->pieces[i].row = brd->p2->pieces[i].row;
		temp_board->p2->pieces[i].column = brd->p2->pieces[i].column;
	}
	temp_board->p2->regulars = NULL;
	temp_board->p2->jumps = NULL;
	temp_board->p2->user = brd->p2->user;
	temp_board->turn = brd->turn;
	temp_board->dificulty = brd->dificulty;

	/* Check for a winner. */
	state = comp_turn(temp_board, mov, &utility);
	if (state){		/* Winning the game has good utility. Sooner moves get better utility. */
		utility += 20*(state*temp_board->turn)*temp_board->dificulty;		/* If the winner is this turn's player - Good utility. If not - Bad utility. */
		return utility;		/* If a game ended, no further moves are needed. */
	}

	temp_board->dificulty--;	/* Turns left to "predict" counter. */
	temp_board->turn *= -1;		/* Give turn to the other player. */

	/* Desides player depending on turn. */
	if (temp_board->turn == temp_board->p1->color)
	{
		pl = temp_board->p1;
	}
	else{
		pl = temp_board->p2;
	}

	calculate_all(pl, temp_board);

	/* Get movevements to check, start with jumps. */
	if (pl->jumps != NULL){
		current = pl->jumps;
	}
	else{
		current = pl->regulars;
	}

	/* Get utility for every possible move. */
	while (current != NULL){	
		utility += calc_utility(current->data, temp_board);	/* Calculates utility for the next turn. */

		current = current->next;
	}
	temp_board->dificulty++;	/* Restores counter for future use. */

	destroy_player(temp_board->p1);
	destroy_player(temp_board->p2);
	destroy_board(temp_board);

	return utility;

} /*calc_utility*/

/* Gets a board and calculates the best movement possible for a computer player, depending on dificulty. */
movement_t	best_move(board brd){

	list		current;
	player		pl;
	movement_t	best_move;
	int			utility, max_utility;
	
	/* Desides player depending on turn. */
	if (brd->turn == brd->p1->color)
	{
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}

	/* Get movevements to check, start with jumps. */
	if (pl->jumps != NULL){
		current = pl->jumps;
	}
	else{
		current = pl->regulars;
	}

	best_move = current->data;							/* Prevents value not returned when dificulty = 0. */
	max_utility = calc_utility(current->data, brd);		/* Gets utility for the first move. */
	current = current->next;

	/* Get utility for every possible move. */
	while (current != NULL){
		utility = calc_utility(current->data, brd);
		if ((utility == max_utility) && random_num(2)){		/* Gives an equal chance for two equally good moves to prevent static games. Random = 0 or 1. */
			best_move = current->data;
		}
		if ((utility > max_utility) && random_num(10)){		/* If a better move is found, remember the better move. There's a 1:10 chance of not doing the best move to prevent endless games. */
			max_utility = utility;
			best_move = current->data;
		}

		current = current->next;
	}

	return best_move;

} /*best_move*/

/*---------------------*/
/* Game Play Functions */
/*---------------------*/
/* ~ Internal ~ */
/*--------------*/
/* Gets a movement and a board and executes the movement. */
int			execute(movement_t mov, board brd, char* filename, gamelog log){

	player		pl;
	location_t	pos;
	int			m[4], removed[2], i, special;

	/* m gets coordinations of the movement. */
	m[0] = (mov.origin.row - 'A');
	m[1] = (mov.origin.column - '1');
	m[2] = (mov.destination.row - 'A');
	m[3] = (mov.destination.column - '1');

	/* Moves piece from origin to destination and resets origin. */
	brd->playing_b[m[2]][m[3]] = brd->playing_b[m[0]][m[1]];
	brd->playing_b[m[0]][m[1]].pl = 0;
	brd->playing_b[m[0]][m[1]].king = 0;
	brd->playing_b[m[0]][m[1]].sign = ':';

	/* Changes piece's location on the player's pieces' list. */
	if (brd->p1->color == brd->playing_b[m[2]][m[3]].pl){
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}
	for (i = 0; i < MAX_PIECES; i++){
		if ((pl->pieces[i].row == (m[0] + 'A')) && (pl->pieces[i].column == (m[1] + '1'))){
			pl->pieces[i].row = (m[2] + 'A');
			pl->pieces[i].column = (m[3] + '1');
			break;
		}
	}

	/* Updates log. */
	if (log->head == NULL){
		log->head = create_list(mov);
		log->tail = log->head;
	}
	else{
		log->tail = add_movement(log->tail, mov);
	}

	/* Special actions after jump. */
	if (mov.type == jump){

		/* Gets the location for the removed piece. */
		if (m[2] > m[0]){
			removed[0] = (m[2] - 1);
		}
		else{
			removed[0] = (m[2] + 1);
		}
		if (m[3] > m[1]){
			removed[1] = (m[3] - 1);
		}
		else{
			removed[1] = (m[3] + 1);
		}

		/* Removes piece from player's list. */
		if (brd->p1->color == brd->playing_b[removed[0]][removed[1]].pl){
			pl = brd->p1;
		}
		else{
			pl = brd->p2;
		}
		for (i = 0; i < MAX_PIECES; i++){
			if ((pl->pieces[i].row == (removed[0] + 'A')) && (pl->pieces[i].column == (removed[1] + '1'))){
				reset_location(pl->pieces+i);
				break;
			}
		}

		/* Resets the removed piece location. */
		brd->playing_b[removed[0]][removed[1]].pl = 0;
		brd->playing_b[removed[0]][removed[1]].king = 0;
		brd->playing_b[removed[0]][removed[1]].sign = ':';

		/* Checks for consecutive jumps.*/
		if (brd->p1->color == brd->playing_b[m[2]][m[3]].pl){
			pl = brd->p1;
		}
		else{
			pl = brd->p2;
		}
		pos.row = mov.destination.row;
		pos.column = mov.destination.column;
		pl->jumps = jump_again(&pos, brd);
		pl->regulars = NULL;
		if (pl->jumps){		/* If exist, gets move from player. */
			switch (pl->user){
				case human:{
					special = get_movement(brd, &mov, log);
					if (special){
						return special;
					}
					break;
						   }
				case computer:{
					system("clear");		/* Clear screen. */
					print_turn(brd);
					print_board(brd);
					printf("Thinking...");
					mov = best_move(brd);
					break;
							  }
				case file:{
					special = get_move_f(brd, &mov, filename);
					if (special){
						return special;
					}
					break;
						  }
			}

			/* Executes consecutive jump. */
			execute(mov, brd, filename, log);

		}

	}

	/* If piece is not a king, and moving towards opponent will exceed the board - Piece gets king. */
	if (!brd->playing_b[m[2]][m[3]].king && (((m[2] + (1*mov.color)) < 0) || ((m[2] + (1*mov.color)) > (BOARD_SIZE - 1)))){
		brd->playing_b[m[2]][m[3]].king = 1;
		brd->playing_b[m[2]][m[3]].sign = upper_char(brd->playing_b[m[2]][m[3]].sign);
		printf("<< KINGED >>\n");
	}

	return 0;

} /*execute*/

/*--------------*/
/* ~ External ~ */
/*--------------*/
void		init(board brd, player p1, player p2){

	int i, j, p_index = 0;

	p1->color = black;		/* Player 1 is black. */
	p1->regulars = NULL;	/* A player has no moves before game-start. */
	p1->jumps = NULL;		/* A player has no jumps before game-start. */
	p2->color = white;		/* Player 2 is white. */
	p2->regulars = NULL;	/* A player has no moves before game-start. */
	p2->jumps = NULL;		/* A player has no jumps before game-start. */

	brd->p1 = p1;		/* Gets player 1. */
	brd->p2 = p2;		/* Gets player 2. */

	for (i = 0; i < STARTING_NUM_ROWS; i++){		/* Initializes the 'black' player's pieces. */
		for (j = 0; j < BOARD_SIZE; j++){
			if ((i%2) != (j%2)){					/* Places pieces only on 'black' squares. */
				brd->playing_b[i][j].sign = 'b';
				brd->playing_b[i][j].king = 0;
				brd->playing_b[i][j].pl = p1->color;
				brd->p1->pieces[p_index].row = (i + 'A');
				brd->p1->pieces[p_index].column = (j + '1');
				p_index++;
			}
		}
	}

	for (; i <BOARD_SIZE -STARTING_NUM_ROWS; i++){
		for (j = 0; j < BOARD_SIZE; j++){			/* Initializes the middle from where i stopped. */
			brd->playing_b[i][j].pl = 0;
			brd->playing_b[i][j].king = 0;
			if ((i%2) == (j%2)){
				brd->playing_b[i][j].sign = ' ';	/* When both are even or odd - White square. */
			}
			else{
				brd->playing_b[i][j].sign = ':';	/* When i and j differ - Black square. */
			}
		}
	}

	p_index = 0;
	for (; i < BOARD_SIZE; i++){					/* Initializes the 'white' player's pieces. */
		for (j = 0; j < BOARD_SIZE; j++){
			if ((i%2) != (j%2)){					/* Places pieces only on 'black' squares. */
				brd->playing_b[i][j].sign = 'w';
				brd->playing_b[i][j].king = 0;
				brd->playing_b[i][j].pl = p2->color;
				brd->p2->pieces[p_index].row = (i + 'A');
				brd->p2->pieces[p_index].column = (j + '1');
				p_index++;
			}
		}
	}

} /*init*/

void		print_board(board brd){

	int	i, j;

	printf("   ");
	for (i = 0; i < BOARD_SIZE; i++){
		printf(" %d", (i+1));
	}
	printf("\n   ");
	for (i = 0; i < BOARD_SIZE; i++){
		printf("+-");
	}
	printf("+");

	for (i = 0; i < BOARD_SIZE; i++){
		printf("\n %c ", (i+'A'));
		for (j = 0; j < BOARD_SIZE; j++){
			printf("|%c", brd->playing_b[i][j].sign);
		}
		printf("|\n   ");
		for (j = 0; j < BOARD_SIZE; j++){
			printf("+-");
		}
		printf("+");
	}

	printf("\n\n");

} /*print_board*/

void		print_log(gamelog log){

	list	current = log->head;
	char	color[COLOR_NAME_SIZE + 1];

	while (current != NULL){
		if (current->data.color == 1){
			str_copy(color, "Black");
		}
		else{
			str_copy(color, "White");
		}

		printf("%s: %c%c to %c%c.\n", color, current->data.origin.row, current->data.origin.column, current->data.destination.row, current->data.destination.column);

		current = current->next;
	}

	printf("Press ENTER to continue...\n");
	FLUSH_INPUT;
	system("clear");		/* Clear screen. */

} /*print_log*/

int			turn(board brd, gamelog log, char* filename){

	player		pl;
	movement_t	mov;
	int			special, state;

	/* Desides player depending on turn. */
	if (brd->turn == brd->p1->color)
	{
		pl = brd->p1;
	}
	else{
		pl = brd->p2;
	}

	/* Check for all movements of this player and checks if there is a winner. */
	state = calculate_all(pl, brd);

	/* If no winner, get and execute movement. */
	if (!state){
		switch (pl->user){
		case human:{
			special = get_movement(brd, &mov, log);
			if (special){
				return special;
			}
			break;
				   }
		case computer:{		/* Checks jumps first to prevent illegal movement. */
			system("clear");		/* Clear screen. */
			print_turn(brd);
			print_board(brd);
			printf("Thinking...\n");
			mov = best_move(brd);
			break;
					  }
		case file:{
			special =  get_move_f(brd, &mov, filename);
			if (special){
				return special;
			}
			break;
				  }
		}

		special = execute(mov, brd, filename, log);
		if (special){
			return special;
		}
	}
	else{		/* IF there is a winner, check the file for aditional instructions. */
		if(pl->user == file){
			do{		/* Runs wile the file hasn't ended. */
				special =  get_move_f(brd, &mov, filename);

				switch(special){
					case  99:		/* If theres a special action, return special action's number. */
					case  98:{
						return special;
							 }
					case  97:		/* If there's a movement instruction after a winner was decided, return "playing after done". */
					case  96:
					case   0:{
						return 95;
							 }
					case  91:{		/* Print board if instructed. */
						print_board(brd);
						break;
							 }
					case  92:{		/* Print log if instructed. */
						print_log(log);
						break;
							 }
					case 100:{		/* If EOF is reached, end file read. */
						break;
							 }
				}

			} while (special != 100);
		}
	}
	
	brd->turn *= -1;	/* When a turn is done, pass turn to other player. */

	return state;

} /*play_turn*/